/*
カメラの位置を示すgnuplot用データを出力するプログラム
2006.1.15 F
入力ファイル           出力ファイル
t.dat 並進ベクトル     data/source.dat
r.dat 回転行列        data/shift.dat
                     data/tip.dat
		     gplot.dat
*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "myfunc.c"

#define init 100.0 //カメラモデル：四角錐の高さ[mm]

void mult(int ,int ,int ,int ,double [][N],double [][N],double [][N]);
void trans(int,int,double[][N],double[][N]);
int fin_r(double [][N],int);

main()
{
  int i,j,k,l;
  double r[N][N],    //回転ベクトル
    r_inv[N][N],     //回転ベクトルの逆行列(転置行列に等しい)
    //         t[N][N],    //並進ベクトル(カメラ座標)
    hei[N][N],       //並進ベクトル(ワールド座標)
    tmp1[N][N],      //作業配列
    tmp2[N][N];      //作業配列

  //カメラモデル形状維持配列(四角錐)
  double pos[][N]={ { init, init, init*2.0},
		    { -init, init, init*2.0},
		    { -init, -init, init*2.0},
		    { init, -init, init*2.0},
		    { init, init, init*2.0}};
  int cam;  //座標計算カメラ番号

  FILE *fp1,*fp2,*fp3,*fp4;

  fp1 = fopen("data/source.dat","w+"); //カメラモデルを原点で表示するベクトル
  fp2 = fopen("data/tip.dat","w+");    //カメラモデルの先の座標
  fp3 = fopen("data/shift.dat","w+");  //カメラモデルのポジション(並進ベクトル)
  fp4 = fopen("gplot.dat","w+");    //GNUPLOT出力ファイル
  fprintf(fp4,"#X         Y         Z\n");//生成ファイル用コメント

  for(l=0;l<3;l++){
    cam = l;//読み込みカメラ番号指定
    fin_r(r,cam);//rをファイルから読み込み
    fin_hei(hei,cam);//heiをファイルから読み込み

    trans(3,3,r,r_inv);           //rの逆行列を計算

    //並進ベクトルを出力
    for(k =0; k < 5; k++){
      for(i = 0; i < 3; i++){
	for(j = 0; j < 1; j++){
	  //	    fprintf(fp3,"%10f ",hei[i][j]);
	  fprintf(fp4,"%10f ",hei[i][j]);
        }
      }
      fprintf(fp4,"\n",hei[i][j]);
    }
    //  printf("並進ベクトル %f %f %f\n",hei[0][0],hei[1][0],hei[2][0]);

    for(k=0;k<5;k++){


      //      fprintf(fp3,"\n");
      fprintf(fp4,"\n");

      //並進ベクトルを出力
      for(i = 0; i < 3; i++){
	for(j = 0; j < 1; j++){
	    fprintf(fp3,"%10f ",hei[i][j]);
        }
      }
      fprintf(fp3,"\n");

      //形状データを読み込み
      tmp1[0][0]=pos[k][0];
      tmp1[1][0]=pos[k][1];
      tmp1[2][0]=pos[k][2];

      //形状データを回転方向に向ける
      mult(3,3,3,1,r_inv,tmp1,tmp2);
      for(i = 0; i < 3; i++){
	for(j = 0; j < 1; j++){
	  //	printf("%10f ",tmp2[i][j]);
	  fprintf(fp1,"%10f ",tmp2[i][j]);
	}
      }
      fprintf(fp1,"\n");

      //方向に並進分を加算
      tmp1[0][0] = hei[0][0]+tmp2[0][0];
      tmp1[1][0] = hei[1][0]+tmp2[1][0];
      tmp1[2][0] = hei[2][0]+tmp2[2][0];

      //    printf("並進分加算済み\n")
      for(i = 0; i < 3; i++){
	for(j = 0; j < 1; j++){
	  fprintf(fp2,"%10f ",tmp1[i][j]);
	  fprintf(fp4,"%10f ",tmp1[i][j]);
	}
      }
      fprintf(fp2,"\n");
    }
    fprintf(fp1,"\n");
    fprintf(fp2,"\n");
    fprintf(fp3,"\n");
    fprintf(fp4,"\n\n\n");

  }
  fclose(fp1);
  fclose(fp2);
  fclose(fp3);
  fclose(fp4);
}

void mult(int m1,int n1,int m2,int n2,double a[][N],double b[][N],double c[][N]){
  //arg[五]と[六]の行列のかけ算。(どちらも0行0列より)[一]×[二]行列、[三]、[四]行列をかける。
  int i,j,k;

  //  printf("multiplying [%d][%d] * [%d][%d] : [%d][%d]\n",m1,n1,m2,n2,m1,n2);
  if(n1 != m2){
    printf("Worning n1 != n2 \n");
  }

  //初期化
  for( i = 0; i < m1; i++ ){
    for( j = 0;j < n2; j++ ){
      c[i][j] = 0.0;
    }
  }
  //かけ算
  for( i = 0; i < m1; i++ ){
    for( j = 0;j < n2; j++ ){
      for( k = 0; k < m2 ; k++ ){
	//	printf("%f * %f ,",a[i][k],b[k][j]);
	c[i][j] += a[i][k] * b[k][j];
      } 
      //      printf("\n");
    }
  }
}

void trans(int m,int n,double a[][N],double b[][N])
{
  int i,j;

  for(i = 0; i < m;i++){
    for(j = 0; j < n; j++){
      b[j][i] = a[i][j];
    }
  }
}

int fin_r(double r[][N],int cam_num)
{
  FILE *fpin;//,*fpout;
  char infile[40];//,outfile[40];
  double a,b,c;
  int i,j;

  fpin = fopen("r.dat","r");

  if(fpin==NULL){
    printf("file not found.\n");
    exit(1);
  }else{
      //ダミー格納
    for(j=0;j<cam_num*3;j++){
      fscanf(fpin,"%lf,%lf,%lf\n",&r[0][0],&r[0][1],&r[0][2]);
    }
    for( i=0; i<3; i++){
      fscanf(fpin,"%lf,%lf,%lf\n",&r[i][0],&r[i][1],&r[i][2]);
    }
  }
  fclose(fpin);
  return 0;
}

int fin_hei(double t[][N],int cam_num){

  FILE *fpin;//,*fpout;
  int i,j;

  fpin = fopen("t.dat","r");

  if(fpin==NULL){
    printf("file not found.\n");
    exit(1);
  }else{
      //ダミー格納
    for(j=0;j<cam_num;j++){
      fscanf(fpin,"%lf,%lf,%lf\n",&t[0][0],&t[0][1],&t[0][2]);
    }
    fscanf(fpin,"%lf,%lf,%lf\n",&t[0][0],&t[1][0],&t[2][0]);
  }
  fclose(fpin);
  return 0;
}
